function [x_next, data_next, eval_Func, f_hist, x_hist] = step_nelder_mead(f_current, x_current, data_current, nm_mode, Log, kelley_restart, kelley_alpha, rel_size)

[nargout, nargin] = argn();

if size(x_current,2)==size(x_current,1)+1 then
  n = size(x_current,1);
  init_simplex_good = %T;
else
  n = length(x_current);
  init_simplex_good = %F;
end

if nm_mode=='build_simplex' then
  if ~isdef('rel_size','local') then
    rel_size = 0.1*ones(size(x_current,1),size(x_current,2));
  end

  // We call the init function
  x_current = nm_init(x_current,rel_size);
  x_next    = x_current;
  eval_Func = 0;
  f_hist = [];
  x_hist = [];
  return;
end

if (~isdef('Log','local')) then
  Log = %F;
end
if (~isdef('kelley_restart','local')) then
  kelley_restart = %F;
end
if (~isdef('kelley_alpha','local')) then
  kelley_alpha = 1e-4;
end

f_hist_is_defined = (nargout>=4);
x_hist_is_defined = (nargout==5);

Compute = %F;
// Initialisation of the mlist data_next structure

data_next = mlist(['psimplex','f_Worth','f_Good','f_Best','x_Worth','x_Good','x_Best','Case','f_Reflexion','x_Reflexion','f_Expansion','x_Expansion', ...
                  'f_C1','x_C1','f_C2','x_C2','f_Contr','x_Contr','f_Shrink','x_Shrink','f_Median','x_Median','f_Hist','x_Hist','eval_Func', ...
                  'f_Simplex','x_Simplex']);

while ~Compute
  if (nm_mode=='init') then
    n = size(x_current,1);
    if (size(x_current,2)~=n+1) then
      printf('nelder_mead: you must give %d starting points\n',n+1);
      abort;
    end
    // Initialisation of the state structure
    data_next('f_Simplex') = list();
    data_next('x_Simplex') = list();
    for i=1:n+1
      data_next('f_Simplex')(i) = f_current(i);
      data_next('x_Simplex')(i) = x_current(:,i);
    end

    // We order the values of the simplex
    [f_sort, I] = sort(f_current);

    data_next('f_Worth') = f_current(I(1));
    data_next('f_Good')  = f_current(I($-1));
    data_next('f_Best')  = f_current(I($));
    data_next('x_Worth') = x_current(:,I(1));
    data_next('x_Good')  = x_current(:,I($-1));
    data_next('x_Best')  = x_current(:,I($));
    data_next('Case')    = 0;  // Case
    data_next('f_Reflexion') = -1;  // f_Reflexion
    data_next('x_Reflexion') = []; // x_Reflexion
    data_next('f_Expansion') = -1;  // f_Expansion
    data_next('x_Expansion') = []; // x_Expansion
    data_next('f_C1') = -1;  // f_C1
    data_next('x_C1') = []; // x_C1
    data_next('f_C2') = -1;  // f_C2;
    data_next('x_C2') = []; // x_C2
    data_next('f_Contr') = -1;  // f_Contr;
    data_next('x_Contr') = []; // x_Contr;
    data_next('f_Shrink') = -1;  // f_Shrink
    data_next('x_Shrink') = []; // x_Shrink
    data_next('f_Median') = -1;  // f_Median
    data_next('x_Median') = []; // x_Median
    data_next('f_Hist') = list(); // f_hist
    data_next('x_Hist') = list(); // x_hist
    data_next('eval_Func') = 0; // eval_Func

    if (x_hist_is_defined) then
      data_next('x_Hist')($+1)  = list();
      data_next('x_Hist')($)(1) = data_next('x_Best');
      data_next('x_Hist')($)(2) = data_next('x_Good');
      data_next('x_Hist')($)(3) = data_next('x_Worth');
    end

    if (f_hist_is_defined) then
      data_next('f_Hist')($+1)  = list();
      data_next('f_Hist')($)(1) = data_next('f_Best');
      data_next('f_Hist')($)(2) = data_next('f_Good');
      data_next('f_Hist')($)(3) = data_next('f_Worth');
    end

    data_current = mlist(['psimplex','f_Worth','f_Good','f_Best','x_Worth','x_Good','x_Best','Case','f_Reflexion','x_Reflexion','f_Expansion','x_Expansion', ...
                          'f_C1','x_C1','f_C2','x_C2','f_Contr','x_Contr','f_Shrink','x_Shrink','f_Median','x_Median','f_Hist','x_Hist','eval_Func', ...
                          'f_Simplex','x_Simplex']);

    f_Reflexion = -1;
    x_Reflexion = [];
    f_Expansion = -1;
    x_Expansion = [];
    f_C1        = -1;
    x_C1        = [];
    f_C2        = -1;
    x_C2        = [];
    f_Contr     = -1;
    x_Contr     = [];
    f_Shrink    = -1;
    x_Shrink    = [];
    f_Median    = -1;
    x_Median    = [];

    f_Worth = data_next('f_Worth');
    f_Good  = data_next('f_Good');
    f_Best  = data_next('f_Best');
    x_Worth = data_next('x_Worth');
    x_Good  = data_next('x_Good');
    x_Best  = data_next('x_Best');

    Case        = 1; // Starting position of the algorithm
  end

  if (nm_mode=='run') then
    data_next = mlist(['psimplex','f_Worth','f_Good','f_Best','x_Worth','x_Good','x_Best','Case','f_Reflexion','x_Reflexion','f_Expansion','x_Expansion', ...
                       'f_C1','x_C1','f_C1','x_C2','f_Contr','x_Contr','f_Shrink','x_Shrink','f_Median','x_Median','f_Hist','x_Hist','eval_Func', ...
                       'f_Simplex','x_Simplex']);

    f_Worth = data_current('f_Worth');
    f_Good  = data_current('f_Good');
    f_Best  = data_current('f_Best');
    x_Worth = data_current('x_Worth');
    x_Good  = data_current('x_Good');
    x_Best  = data_current('x_Best');

    f_Reflexion = data_current('f_Reflexion');
    x_Reflexion = data_current('x_Reflexion');
    f_Expansion = data_current('f_Expansion');
    x_Expansion = data_current('x_Expansion');
    f_C1        = data_current('f_C1');
    x_C1        = data_current('x_C1');
    f_C2        = data_current('f_C2');
    x_C2        = data_current('x_C2');
    f_Contr     = data_current('f_Contr');
    x_Contr     = data_current('x_Contr');
    f_Shrink    = data_current('f_Shrink');
    x_Shrink    = data_current('x_Shrink');
    f_Median    = data_current('f_Median');
    x_Median    = data_current('x_Median');

    eval_Func   = data_current('eval_Func');

    Case        = data_current('Case');

    data_next   = data_current;
  end

  if (nm_mode=='exit') then
    x_next    = data_current('x_Best');
    data_next = data_current('f_Best');
    f_hist    = data_current('f_Hist');
    x_hist    = data_current('x_Hist');
    return;
  end

  if (Log) then
    printf('step_nelder_mead: initialization\n');
    printf('f_Worth = %f f_Good = %f f_Best = %f\n', f_Worth, f_Good, f_Best);
  end

  select(Case)
  case 1 then
    // Computation of R
    x_Median    = (x_Best + x_Good) / 2.0
    x_Reflexion = 2.0 * x_Median - x_Worth;
    x_next      = x_Reflexion;
    data_next('x_Reflexion') = x_Reflexion;
    data_next('x_Median')    = x_Median;
    data_next('Case')        = 2; // Go to block 2
    Compute                  = %T; // Ask the user for an objective function value
  case 2 then
    // We get f(R)
    f_Reflexion              = f_current;
    data_next('f_Reflexion') = f_Reflexion;
    data_next('Case')        = 3; // Go to block 3
    eval_Func                = eval_Func + 1;
    data_next('eval_Func')   = eval_Func;
    Compute                  = %F;
  case 3 then
    if (f_Reflexion<f_Good) then
      data_next('Case') = 4; // Go to block 4
    else
      data_next('Case') = 11; // Go to block 11
    end
    Compute = %F;
  case 4 then
    if (f_Best<f_Reflexion) then
      data_next('Case') = 5; // Go to block 5
    else
      data_next('Case') = 6; // Go to block 6
    end
    Compute = %F;
  case 5 then
    f_Worth  = f_Reflexion;
    x_Worth  = x_Reflexion;
    data_next('f_Worth')  = f_Worth;
    data_next('x_Worth')  = x_Worth;
    data_next('Case')     = 22; // Next Case (we start again), but before, we rank the points
    Compute = %F;
  case 6 then
    // Computation of E
    x_Expansion   = 2.0 * x_Reflexion - x_Median;
    x_next        = x_Expansion;
    data_next('x_Expansion') = x_Expansion;
    data_next('Case')        = 7; // Go to block 7
    Compute                  = %T;
  case 7 then
    // Computation of f(E)
    f_Expansion   = f_current;
    data_next('f_Expansion') = f_Expansion;
    data_next('Case')        = 8; // Go to block 8
    eval_Func                = eval_Func + 1;
    data_next('eval_Func')   = eval_Func;
    Compute                  = %F;
  case 8 then
    if (f_Expansion<f_Best) then
      data_next('Case') = 9; // Go to block 9
    else
      data_next('Case') = 10; // Go to block 10
    end
    Compute = %F;
  case 9 then
    // We replace W by E
    x_Worth      = x_Expansion;
    f_Worth      = f_Expansion;
    data_next('f_Worth') = f_Worth;
    data_next('x_Worth') = x_Worth;
    data_next('Case')    = 22; // Next Case: we start again but before, we rank the points
    Compute              = %F;
  case 10 then
    // We replace W by R
    x_Worth       = x_Reflexion;
    f_Worth       = f_Reflexion;
    data_next('f_Worth') = f_Worth;
    data_next('x_Worth') = x_Worth;
    data_next('Case')    = 22; // Go to block 22
    Compute              = %F;
  case 11 then
    // if f(R) < f(W)
    if (f_Reflexion<f_Worth) then
      data_next('Case') = 12;
    else
      data_next('Case') = 13;
    end
    Compute = %F;
  case 12 then
    // We replace W by R
    x_Worth      = x_Reflexion;
    f_Worth      = f_Reflexion;
    data_next('f_Worth') = f_Worth;
    data_next('x_Worth') = x_Worth;
    data_next('Case')    = 13; // Go to block 13
    Compute              = %F;
  case 13 then
    // Computation of C1
    x_C1          = (x_Median + x_Worth) / 2.0;
    x_next        = x_C1;
    data_next('x_C1') = x_C1;
    data_next('Case') = 14; // Go to block 14
    Compute = %T;
  case 14 then
    // Computation of f(C1)
    f_C1  = f_current;
    data_next('f_C1')      = f_C1;
    data_next('Case')      = 15; // Go to block 15
    eval_Func              = eval_Func + 1;
    data_next('eval_Func') = eval_Func;
    Compute                = %F;
  case 15 then
    // Computation of C2
    x_C2   = (x_Median + x_Reflexion) / 2.0;
    x_next = x_C2;
    data_next('x_C2') = x_C2;
    data_next('Case') = 16; // Go to block 16
    Compute           = %T;
  case 16 then
    // Computation of f(C2)
    f_C2 = f_current;
    data_next('f_C2')      = f_C2;
    data_next('Case')      = 17;
    eval_Func              = eval_Func + 1;
    data_next('eval_Func') = eval_Func;
    Compute                = %F;
  case 17 then
    if (f_C1<f_C2) then
      f_Contr = f_C1;
      x_Contr = x_C1;
    else
      f_Contr = f_C2;
      x_Contr = x_C2;
    end
    data_next('f_Contr') = f_Contr;
    data_next('x_Contr') = x_Contr;
    if (f_Contr<f_Worth) then
      data_next('Case') = 21; // Go to block 21
    else
      data_next('Case') = 18; // Go to block 18
    end
    Compute = %F;
  case 18 then
    // Computation of S
    x_Shrink = (x_Best + x_Worth) / 2.0;
    x_next   = x_Shrink;
    data_next('x_Shrink') = x_Shrink; 
    data_next('Case')     = 19; // Go to block 19
    Compute               = %T;
  case 19 then
    // Computation of f(S)
    f_Shrink = f_current;
    data_next('f_Shrink')  = f_Shrink;
    data_next('Case')      = 20; // Go to block 20
    eval_Func              = eval_Func + 1;
    data_next('eval_Func') = eval_Func;
    Compute                = %F;
  case 20 then
    // We replace W by S and G by M
    x_Worth = x_Shrink;
    f_Worth = f_Shrink;
    x_Good  = x_Median;
    f_Good  = f_Median;
    data_next('f_Good')  = f_Good;
    data_next('f_Worth') = f_Worth;
    data_next('x_Good')  = x_Good;
    data_next('x_Worth') = x_Worth;
    data_next('Case')    = 22; // Go to block 22
    Compute              = %F;
  case 21 then
    // We replace W by C
    x_Worth = x_Contr;
    f_Worth = f_Contr;
    data_next('f_Worth') = f_Worth;
    data_next('x_Worth') = x_Worth;
    data_next('Case')    = 22; // Go to block 22
    Compute              = %F;
  case 22 then
    // Ranking the points
    f_nm = [];
    x_nm = list();
    for i=1:n+1
      f_nm(i)   = data_next('f_Simplex')(i);
      x_nm($+1) = data_next('x_Simplex')(i);
    end
    f_nm(1)   = f_Worth;
    f_nm($-1) = f_Good;
    f_nm($)   = f_Best;

    x_nm(1)   = x_Worth;
    x_nm($-1) = x_Good;
    x_nm($)   = x_Best;

    [f_sort, I] = sort(f_nm);

    f_Worth = f_nm(I(1));
    f_Good  = f_nm(I($-1));
    f_Best  = f_nm(I($));

    x_Worth = x_nm(I(1));
    x_Good  = x_nm(I($-1));
    x_Best  = x_nm(I($));

    // Updating the simplex
    for i=1:n+1
      data_next('f_Simplex')(i) = f_nm(i);
      data_next('x_Simplex')(i) = x_nm(i);
    end
    
    data_next('f_Worth') = f_Worth;
    data_next('f_Good')  = f_Good;
    data_next('f_Best')  = f_Best;
    data_next('x_Worth') = x_Worth;
    data_next('x_Good')  = x_Good;
    data_next('x_Best')  = x_Best;
    
    if (x_hist_is_defined) then
      data_next('x_Hist')($+1)  = list();
      data_next('x_Hist')($)(1) = x_Best;
      data_next('x_Hist')($)(2) = x_Good;
      data_next('x_Hist')($)(3) = x_Worth;
    end

    if (f_hist_is_defined) then
      data_next('f_Hist')($+1)  = list();
      data_next('f_Hist')($)(1) = f_Best;
      data_next('f_Hist')($)(2) = f_Good;
      data_next('f_Hist')($)(3) = f_Worth;
    end

    data_next('Case') = 1; // Go back to block 1
    Compute           = %F;
  case 23 then
    // Computation of M
    x_next = x_Median;
    data_next('x_Median') = x_Median;
    data_next('Case')     = 24; // Go to block 24
    Compute               = %T;
  case 24 then
    // Computation of f(M)
    f_Median = f_current;
    data_next('f_Median')  = f_Median;
    data_next('Case')      = 20; // Go to block 20
    eval_Func              = eval_Func + 1;
    data_next('eval_Func') = eval_Func;
    Compute                = %F;
  end
  
  data_current = data_next; // when in the while loop, we update the data_current
                            // structure to update the state of the Nelder and Mead
                            // method
end

if (f_hist_is_defined) then
  f_hist = data_next('f_Hist');
end
if (x_hist_is_defined) then
  x_hist = data_next('x_Hist');
end
eval_Func = data_next('eval_Func');
endfunction

